@@ -1,13 +1,20 @@
+import six
 import datetime
 
+import django
 from django.db import models
 from django import test
 from django.utils.functional import curry
+from django.utils.translation import activate, deactivate
 
 import dbsettings
 from dbsettings import loading, views
 
+
 # Set up some settings to test
+MODULE_NAME = 'dbsettings.tests.tests'
+
+
 class TestSettings(dbsettings.Group):
     boolean = dbsettings.BooleanValue()
     integer = dbsettings.IntegerValue()
@@ -19,7 +26,8 @@ class TestSettings(dbsettings.Group):
     datetime = dbsettings.DateTimeValue()
 
 # This is assigned to module, rather than a model
-module_settings = TestSettings()
+module_settings = TestSettings(app_label='dbsettings')
+
 
 class Defaults(models.Model):
     class settings(dbsettings.Group):
@@ -27,83 +35,113 @@ class Defaults(models.Model):
         boolean_false = dbsettings.BooleanValue(default=False)
         integer = dbsettings.IntegerValue(default=1)
         string = dbsettings.StringValue(default="default")
-        list_semi_colon = dbsettings.MultiSeparatorValue(default=['one','two'])
-        list_comma = dbsettings.MultiSeparatorValue(separator=',',default=('one','two'))
+        list_semi_colon = dbsettings.MultiSeparatorValue(default=['one', 'two'])
+        list_comma = dbsettings.MultiSeparatorValue(separator=',', default=('one', 'two'))
         date = dbsettings.DateValue(default=datetime.date(2012, 3, 14))
         time = dbsettings.TimeValue(default=datetime.time(12, 3, 14))
         datetime = dbsettings.DateTimeValue(default=datetime.datetime(2012, 3, 14, 12, 3, 14))
     settings = settings()
 
+
+class TestBaseModel(models.Model):
+    class Meta:
+        abstract = True
+        app_label = 'dbsettings'
+
+
 # These will be populated by the fixture data
-class Populated(models.Model):
+class Populated(TestBaseModel):
     settings = TestSettings()
 
+
 # These will be empty after startup
-class Unpopulated(models.Model):
+class Unpopulated(TestBaseModel):
     settings = TestSettings()
 
+
 # These will allow blank values
-class Blankable(models.Model):
+class Blankable(TestBaseModel):
     settings = TestSettings()
 
-class Editable(models.Model):
-    settings = TestSettings()
 
-class Combined(models.Model):
+class Editable(TestBaseModel):
+    settings = TestSettings('Verbose name')
+
+
+class Combined(TestBaseModel):
     class settings(dbsettings.Group):
         enabled = dbsettings.BooleanValue()
     settings = TestSettings() + settings()
 
+
 # For registration testing
 class ClashSettings1(dbsettings.Group):
     clash1 = dbsettings.BooleanValue()
 
+
 class ClashSettings2(dbsettings.Group):
     clash2 = dbsettings.BooleanValue()
 
+
 class ClashSettings1_2(dbsettings.Group):
     clash1 = dbsettings.IntegerValue()
     clash2 = dbsettings.IntegerValue()
 
-module_clash1 = ClashSettings1()
+module_clash1 = ClashSettings1(app_label='dbsettings')
 
-class ModelClash(models.Model):
+
+class ModelClash(TestBaseModel):
     settings = ClashSettings1_2()
 
-module_clash2 = ClashSettings2()
+module_clash2 = ClashSettings2(app_label='dbsettings')
 
 
 class SettingsTestCase(test.TestCase):
     urls = 'dbsettings.tests.test_urls'
 
+    @classmethod
+    def setUpClass(cls):
+        # Since some text assertions are performed, make sure that no translation interrupts.
+        activate('en')
+
+    @classmethod
+    def tearDownClass(cls):
+        deactivate()
+
     def setUp(self):
         # Standard test fixtures don't update the in-memory cache.
         # So we have to do it ourselves this time.
-        loading.set_setting_value('dbsettings.tests.tests', 'Populated', 'boolean', True)
-        loading.set_setting_value('dbsettings.tests.tests', 'Populated', 'integer', 42)
-        loading.set_setting_value('dbsettings.tests.tests', 'Populated', 'string', 'Ni!')
-        loading.set_setting_value('dbsettings.tests.tests', 'Populated', 'list_semi_colon', 'a@b.com;c@d.com;e@f.com')
-        loading.set_setting_value('dbsettings.tests.tests', 'Populated', 'list_comma', 'a@b.com,c@d.com,e@f.com')
-        loading.set_setting_value('dbsettings.tests.tests', 'Populated', 'date', '2012-06-28')
-        loading.set_setting_value('dbsettings.tests.tests', 'Populated', 'time', '16:19:17')
-        loading.set_setting_value('dbsettings.tests.tests', 'Populated', 'datetime', '2012-06-28 16:19:17')
-        loading.set_setting_value('dbsettings.tests.tests', '', 'boolean', False)
-        loading.set_setting_value('dbsettings.tests.tests', '', 'integer', 14)
-        loading.set_setting_value('dbsettings.tests.tests', '', 'string', 'Module')
-        loading.set_setting_value('dbsettings.tests.tests', '', 'list_semi_colon', 'g@h.com;i@j.com;k@l.com')
-        loading.set_setting_value('dbsettings.tests.tests', '', 'list_comma', 'g@h.com,i@j.com,k@l.com')
-        loading.set_setting_value('dbsettings.tests.tests', '', 'date', '2011-05-27')
-        loading.set_setting_value('dbsettings.tests.tests', '', 'time', '15:18:16')
-        loading.set_setting_value('dbsettings.tests.tests', '', 'datetime', '2011-05-27 15:18:16')
-        loading.set_setting_value('dbsettings.tests.tests', 'Combined', 'boolean', False)
-        loading.set_setting_value('dbsettings.tests.tests', 'Combined', 'integer', 1138)
-        loading.set_setting_value('dbsettings.tests.tests', 'Combined', 'string', 'THX')
-        loading.set_setting_value('dbsettings.tests.tests', 'Combined', 'list_semi_colon', 'm@n.com;o@p.com;q@r.com')
-        loading.set_setting_value('dbsettings.tests.tests', 'Combined', 'list_comma', 'm@n.com,o@p.com,q@r.com')
-        loading.set_setting_value('dbsettings.tests.tests', 'Combined', 'date', '2010-04-26')
-        loading.set_setting_value('dbsettings.tests.tests', 'Combined', 'time', '14:17:15')
-        loading.set_setting_value('dbsettings.tests.tests', 'Combined', 'datetime', '2010-04-26 14:17:15')
-        loading.set_setting_value('dbsettings.tests.tests', 'Combined', 'enabled', True)
+        loading.set_setting_value(MODULE_NAME, 'Populated', 'boolean', True)
+        loading.set_setting_value(MODULE_NAME, 'Populated', 'integer', 42)
+        loading.set_setting_value(MODULE_NAME, 'Populated', 'string', 'Ni!')
+        loading.set_setting_value(MODULE_NAME, 'Populated', 'list_semi_colon',
+                                  'a@b.com;c@d.com;e@f.com')
+        loading.set_setting_value(MODULE_NAME, 'Populated', 'list_comma',
+                                  'a@b.com,c@d.com,e@f.com')
+        loading.set_setting_value(MODULE_NAME, 'Populated', 'date', '2012-06-28')
+        loading.set_setting_value(MODULE_NAME, 'Populated', 'time', '16:19:17')
+        loading.set_setting_value(MODULE_NAME, 'Populated', 'datetime',
+                                  '2012-06-28 16:19:17')
+        loading.set_setting_value(MODULE_NAME, '', 'boolean', False)
+        loading.set_setting_value(MODULE_NAME, '', 'integer', 14)
+        loading.set_setting_value(MODULE_NAME, '', 'string', 'Module')
+        loading.set_setting_value(MODULE_NAME, '', 'list_semi_colon',
+                                  'g@h.com;i@j.com;k@l.com')
+        loading.set_setting_value(MODULE_NAME, '', 'list_comma', 'g@h.com,i@j.com,k@l.com')
+        loading.set_setting_value(MODULE_NAME, '', 'date', '2011-05-27')
+        loading.set_setting_value(MODULE_NAME, '', 'time', '15:18:16')
+        loading.set_setting_value(MODULE_NAME, '', 'datetime', '2011-05-27 15:18:16')
+        loading.set_setting_value(MODULE_NAME, 'Combined', 'boolean', False)
+        loading.set_setting_value(MODULE_NAME, 'Combined', 'integer', 1138)
+        loading.set_setting_value(MODULE_NAME, 'Combined', 'string', 'THX')
+        loading.set_setting_value(MODULE_NAME, 'Combined', 'list_semi_colon',
+                                  'm@n.com;o@p.com;q@r.com')
+        loading.set_setting_value(MODULE_NAME, 'Combined', 'list_comma',
+                                  'm@n.com,o@p.com,q@r.com')
+        loading.set_setting_value(MODULE_NAME, 'Combined', 'date', '2010-04-26')
+        loading.set_setting_value(MODULE_NAME, 'Combined', 'time', '14:17:15')
+        loading.set_setting_value(MODULE_NAME, 'Combined', 'datetime', '2010-04-26 14:17:15')
+        loading.set_setting_value(MODULE_NAME, 'Combined', 'enabled', True)
 
     def test_settings(self):
         "Make sure settings groups are initialized properly"
@@ -146,18 +184,17 @@ class SettingsTestCase(test.TestCase):
         self.assertEqual(Unpopulated.settings.list_semi_colon, [])
         self.assertEqual(Unpopulated.settings.list_comma, [])
 
-        # ...Unless a default paramter was specified, then they use that
+        # ...Unless a default parameter was specified, then they use that
         self.assertEqual(Defaults.settings.boolean, True)
         self.assertEqual(Defaults.settings.boolean_false, False)
         self.assertEqual(Defaults.settings.integer, 1)
         self.assertEqual(Defaults.settings.string, 'default')
-        self.assertEqual(Defaults.settings.list_semi_colon, ['one','two'])
-        self.assertEqual(Defaults.settings.list_comma, ['one','two'])
+        self.assertEqual(Defaults.settings.list_semi_colon, ['one', 'two'])
+        self.assertEqual(Defaults.settings.list_comma, ['one', 'two'])
         self.assertEqual(Defaults.settings.date, datetime.date(2012, 3, 14))
         self.assertEqual(Defaults.settings.time, datetime.time(12, 3, 14))
         self.assertEqual(Defaults.settings.datetime, datetime.datetime(2012, 3, 14, 12, 3, 14))
 
-
         # Settings should be retrieved in the order of definition
         self.assertEqual(Populated.settings.keys(),
                          ['boolean', 'integer', 'string', 'list_semi_colon',
@@ -167,24 +204,29 @@ class SettingsTestCase(test.TestCase):
                           'list_comma', 'date', 'time', 'datetime', 'enabled'])
 
         # Values should be coerced to the proper Python types
-        self.assert_(isinstance(Populated.settings.boolean, bool))
-        self.assert_(isinstance(Populated.settings.integer, int))
-        self.assert_(isinstance(Populated.settings.string, basestring))
+        self.assertTrue(isinstance(Populated.settings.boolean, bool))
+        self.assertTrue(isinstance(Populated.settings.integer, int))
+        self.assertTrue(isinstance(Populated.settings.string, six.string_types))
 
         # Settings can not be accessed directly from models, only instances
         self.assertRaises(AttributeError, lambda: Populated().settings)
         self.assertRaises(AttributeError, lambda: Unpopulated().settings)
 
         # Updates are reflected in the live settings
-        loading.set_setting_value('dbsettings.tests.tests', 'Unpopulated', 'boolean', True)
-        loading.set_setting_value('dbsettings.tests.tests', 'Unpopulated', 'integer', 13)
-        loading.set_setting_value('dbsettings.tests.tests', 'Unpopulated', 'string', 'Friday')
-        loading.set_setting_value('dbsettings.tests.tests', 'Unpopulated', 'list_semi_colon', 'aa@bb.com;cc@dd.com')
-        loading.set_setting_value('dbsettings.tests.tests', 'Unpopulated', 'list_comma', 'aa@bb.com,cc@dd.com')
+        loading.set_setting_value(MODULE_NAME, 'Unpopulated', 'boolean', True)
+        loading.set_setting_value(MODULE_NAME, 'Unpopulated', 'integer', 13)
+        loading.set_setting_value(MODULE_NAME, 'Unpopulated', 'string', 'Friday')
+        loading.set_setting_value(MODULE_NAME, 'Unpopulated', 'list_semi_colon',
+                                  'aa@bb.com;cc@dd.com')
+        loading.set_setting_value(MODULE_NAME, 'Unpopulated', 'list_comma',
+                                  'aa@bb.com,cc@dd.com')
         # for date/time you can specify string (as above) or proper object
-        loading.set_setting_value('dbsettings.tests.tests', 'Unpopulated', 'date', datetime.date(1912, 6, 23))
-        loading.set_setting_value('dbsettings.tests.tests', 'Unpopulated', 'time', datetime.time(1, 2, 3))
-        loading.set_setting_value('dbsettings.tests.tests', 'Unpopulated', 'datetime', datetime.datetime(1912, 6, 23, 1, 2, 3))
+        loading.set_setting_value(MODULE_NAME, 'Unpopulated', 'date',
+                                  datetime.date(1912, 6, 23))
+        loading.set_setting_value(MODULE_NAME, 'Unpopulated', 'time',
+                                  datetime.time(1, 2, 3))
+        loading.set_setting_value(MODULE_NAME, 'Unpopulated', 'datetime',
+                                  datetime.datetime(1912, 6, 23, 1, 2, 3))
 
         self.assertEqual(Unpopulated.settings.boolean, True)
         self.assertEqual(Unpopulated.settings.integer, 13)
@@ -196,17 +238,16 @@ class SettingsTestCase(test.TestCase):
         self.assertEqual(Unpopulated.settings.datetime, datetime.datetime(1912, 6, 23, 1, 2, 3))
 
         # Updating settings with defaults
-        loading.set_setting_value('dbsettings.tests.tests', 'Defaults', 'boolean', False)
+        loading.set_setting_value(MODULE_NAME, 'Defaults', 'boolean', False)
         self.assertEqual(Defaults.settings.boolean, False)
-        loading.set_setting_value('dbsettings.tests.tests', 'Defaults', 'boolean_false', True)
+        loading.set_setting_value(MODULE_NAME, 'Defaults', 'boolean_false', True)
         self.assertEqual(Defaults.settings.boolean_false, True)
 
-
         # Updating blankable settings
         self.assertEqual(Blankable.settings.string, '')
-        loading.set_setting_value('dbsettings.tests.tests', 'Blankable', 'string', 'Eli')
+        loading.set_setting_value(MODULE_NAME, 'Blankable', 'string', 'Eli')
         self.assertEqual(Blankable.settings.string, 'Eli')
-        loading.set_setting_value('dbsettings.tests.tests', 'Blankable', 'string', '')
+        loading.set_setting_value(MODULE_NAME, 'Blankable', 'string', '')
         self.assertEqual(Blankable.settings.string, '')
 
         # And they can be modified in-place
@@ -244,8 +285,8 @@ class SettingsTestCase(test.TestCase):
         self.assertRaises(TypeError, curry(type, 'BadGroup', (dbsettings.Group,), attrs))
 
         # Make sure affect models get the new permissions
-        self.assert_('can_edit_populated_settings' in dict(Populated._meta.permissions))
-        self.assert_('can_edit_unpopulated_settings' in dict(Unpopulated._meta.permissions))
+        self.assertTrue('can_edit_populated_settings' in dict(Populated._meta.permissions))
+        self.assertTrue('can_edit_unpopulated_settings' in dict(Unpopulated._meta.permissions))
 
     def assertCorrectSetting(self, value_class, *key):
         from dbsettings import loading
@@ -256,10 +297,16 @@ class SettingsTestCase(test.TestCase):
     def test_registration(self):
         "Module and class settings can be mixed up"
         from dbsettings import BooleanValue, IntegerValue
-        self.assertCorrectSetting(BooleanValue, 'dbsettings.tests.tests', '', 'clash1')
-        self.assertCorrectSetting(IntegerValue, 'dbsettings.tests.tests', 'ModelClash', 'clash1')
-        self.assertCorrectSetting(IntegerValue, 'dbsettings.tests.tests', 'ModelClash', 'clash2')
-        self.assertCorrectSetting(BooleanValue, 'dbsettings.tests.tests', '', 'clash2')
+        self.assertCorrectSetting(BooleanValue, MODULE_NAME, '', 'clash1')
+        self.assertCorrectSetting(IntegerValue, MODULE_NAME, 'ModelClash', 'clash1')
+        self.assertCorrectSetting(IntegerValue, MODULE_NAME, 'ModelClash', 'clash2')
+        self.assertCorrectSetting(BooleanValue, MODULE_NAME, '', 'clash2')
+
+    def assertLoginFormShown(self, response):
+        if django.VERSION >= (1, 7):
+            self.assertRedirects(response, '/admin/login/?next=/settings/')
+        else:
+            self.assertTemplateUsed(response, 'admin/login.html')
 
     def test_forms(self):
         "Forms should display only the appropriate settings"
@@ -271,14 +318,17 @@ class SettingsTestCase(test.TestCase):
         # Set up a users to test the editor forms
         user = User.objects.create_user('dbsettings', '', 'dbsettings')
 
+        # Check named url
+        site_form = reverse('site_settings')
+
         # First test without any authenticated user
         response = self.client.get(site_form)
-        self.assertTemplateUsed(response, 'admin/login.html')
+        self.assertLoginFormShown(response)
 
         # Then test a standard non-staff user
         self.client.login(username='dbsettings', password='dbsettings')
         response = self.client.get(site_form)
-        self.assertTemplateUsed(response, 'admin/login.html')
+        self.assertLoginFormShown(response)
 
         # Add staff status, but no settings permissions
         user.is_staff = True
@@ -295,34 +345,45 @@ class SettingsTestCase(test.TestCase):
         perm = Permission.objects.get(codename='can_edit_editable_settings')
         user.user_permissions.add(perm)
 
+        # Check if verbose_name appears
+        response = self.client.get(site_form)
+        self.assertContains(response, 'Verbose name')
+
         # Erroneous submissions should be caught by newforms
         data = {
-            'dbsettings.tests.tests__Editable__integer': '3.5',
-            'dbsettings.tests.tests__Editable__string': '',
-            'dbsettings.tests.tests__Editable__list_semi_colon': '',
-            'dbsettings.tests.tests__Editable__list_comma': '',
-            'dbsettings.tests.tests__Editable__date': '3-77-99',
-            'dbsettings.tests.tests__Editable__time': 'abc',
-            'dbsettings.tests.tests__Editable__datetime': '',
+            '%s__Editable__integer' % MODULE_NAME: '3.5',
+            '%s__Editable__string' % MODULE_NAME: '',
+            '%s__Editable__list_semi_colon' % MODULE_NAME: '',
+            '%s__Editable__list_comma' % MODULE_NAME: '',
+            '%s__Editable__date' % MODULE_NAME: '3-77-99',
+            '%s__Editable__time' % MODULE_NAME: 'abc',
+            '%s__Editable__datetime' % MODULE_NAME: '',
         }
         response = self.client.post(site_form, data)
-        self.assertFormError(response, 'form', 'dbsettings.tests.tests__Editable__integer', 'Enter a whole number.')
-        self.assertFormError(response, 'form', 'dbsettings.tests.tests__Editable__string', 'This field is required.')
-        self.assertFormError(response, 'form', 'dbsettings.tests.tests__Editable__list_semi_colon', 'This field is required.')
-        self.assertFormError(response, 'form', 'dbsettings.tests.tests__Editable__list_comma', 'This field is required.')
-        self.assertFormError(response, 'form', 'dbsettings.tests.tests__Editable__date', 'Enter a valid date.')
-        self.assertFormError(response, 'form', 'dbsettings.tests.tests__Editable__time', 'Enter a valid time.')
-        self.assertFormError(response, 'form', 'dbsettings.tests.tests__Editable__datetime', 'This field is required.')
+        self.assertFormError(response, 'form', '%s__Editable__integer' % MODULE_NAME,
+                             'Enter a whole number.')
+        self.assertFormError(response, 'form', '%s__Editable__string' % MODULE_NAME,
+                             'This field is required.')
+        self.assertFormError(response, 'form', '%s__Editable__list_semi_colon' % MODULE_NAME,
+                             'This field is required.')
+        self.assertFormError(response, 'form', '%s__Editable__list_comma' % MODULE_NAME,
+                             'This field is required.')
+        self.assertFormError(response, 'form', '%s__Editable__date' % MODULE_NAME,
+                             'Enter a valid date.')
+        self.assertFormError(response, 'form', '%s__Editable__time' % MODULE_NAME,
+                             'Enter a valid time.')
+        self.assertFormError(response, 'form', '%s__Editable__datetime' % MODULE_NAME,
+                             'This field is required.')
 
         # Successful submissions should redirect
         data = {
-            'dbsettings.tests.tests__Editable__integer': '4',
-            'dbsettings.tests.tests__Editable__string': 'Success!',
-            'dbsettings.tests.tests__Editable__list_semi_colon': 'jj@kk.com;ll@mm.com',
-            'dbsettings.tests.tests__Editable__list_comma': 'jj@kk.com,ll@mm.com',
-            'dbsettings.tests.tests__Editable__date': '2012-06-28',
-            'dbsettings.tests.tests__Editable__time': '16:37:45',
-            'dbsettings.tests.tests__Editable__datetime': '2012-06-28 16:37:45',
+            '%s__Editable__integer' % MODULE_NAME: '4',
+            '%s__Editable__string' % MODULE_NAME: 'Success!',
+            '%s__Editable__list_semi_colon' % MODULE_NAME: 'jj@kk.com;ll@mm.com',
+            '%s__Editable__list_comma' % MODULE_NAME: 'jj@kk.com,ll@mm.com',
+            '%s__Editable__date' % MODULE_NAME: '2012-06-28',
+            '%s__Editable__time' % MODULE_NAME: '16:37:45',
+            '%s__Editable__datetime' % MODULE_NAME: '2012-06-28 16:37:45',
         }
         response = self.client.post(site_form, data)
         self.assertRedirects(response, site_form)
@@ -336,10 +397,19 @@ class SettingsTestCase(test.TestCase):
         self.assertEqual(Editable.settings.time, datetime.time(16, 37, 45))
         self.assertEqual(Editable.settings.datetime, datetime.datetime(2012, 6, 28, 16, 37, 45))
 
-        # Check if module / class settings show properly
-        #perm = Permission.objects.get(codename='can_edit__settings')  # module-level settings
-        #user.user_permissions.add(perm)
+        # Check if module level settings show properly
+        self._test_form_fields(site_form, 8, False)
+        # Add perm for whole app
+        perm = Permission.objects.get(codename='can_edit__settings')  # module-level settings
+        user.user_permissions.add(perm)
+        self._test_form_fields(site_form, 18)
+        # Remove other perms - left only global perm
+        perm = Permission.objects.get(codename='can_edit_editable_settings')
+        user.user_permissions.remove(perm)
+        self._test_form_fields(site_form, 10)
 
-        #response = self.client.get(site_form)
-        #self.assertTemplateUsed(response, 'dbsettings/site_settings.html')
-        #self.assertEqual(len(response.context[0]['form'].fields), 14)
+    def _test_form_fields(self, url, fields_num, present=True, variable_name='form'):
+        global_setting = '%s____clash2' % MODULE_NAME  # Some global setting name
+        response = self.client.get(url)
+        self.assertEqual(present, global_setting in response.context[0][variable_name].fields)
+        self.assertEqual(len(response.context[0][variable_name].fields), fields_num)
