@@ -12,12 +12,13 @@ from django.conf import settings
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.utils import formats
 from django.utils.safestring import mark_safe
+from django.utils.translation import ugettext_lazy as _
 
 from dbsettings.loading import get_setting_storage, set_setting_value
 
 __all__ = ['Value', 'BooleanValue', 'DecimalValue', 'EmailValue',
            'DurationValue', 'FloatValue', 'IntegerValue', 'PercentValue',
-           'PositiveIntegerValue', 'StringValue', 'TextValue',
+           'PositiveIntegerValue', 'StringValue', 'TextValue', 'PasswordValue',
            'MultiSeparatorValue', 'ImageValue',
            'DateTimeValue', 'DateValue', 'TimeValue']
 
@@ -61,6 +62,10 @@ class Value(object):
 
         setattr(cls, self.attribute_name, self)
 
+    @property
+    def app(self):
+        return getattr(self, '_app', self.module_name.split('.')[-2])
+
     def __get__(self, instance=None, cls=None):
         if instance is None:
             raise AttributeError("%r is only accessible from %s instances." %
@@ -144,7 +149,8 @@ class DurationValue(Value):
             raise forms.ValidationError('The maximum allowed value is %s' % datetime.timedelta.max)
 
     def get_db_prep_save(self, value):
-        return six.text_type(value.days * 24 * 3600 + value.seconds + float(value.microseconds) / 1000000)
+        return six.text_type(value.days * 24 * 3600 + value.seconds
+                             + float(value.microseconds) / 1000000)
 
 
 class FloatValue(Value):
@@ -209,6 +215,23 @@ class EmailValue(Value):
         return six.text_type(value)
 
 
+class PasswordValue(Value):
+    class field(forms.CharField):
+        widget = forms.PasswordInput
+
+        def __init__(self, **kwargs):
+            if not kwargs.get('help_text'):
+                kwargs['help_text'] = _(
+                    'Leave empty in order to retain old password. Provide new value to change.')
+            forms.CharField.__init__(self, **kwargs)
+
+        def clean(self, value):
+            # Retain old password if not changed
+            if value == '':
+                value = self.initial
+            return forms.CharField.clean(self, value)
+
+
 class MultiSeparatorValue(TextValue):
     """Provides a way to store list-like string settings.
     e.g 'mail@test.com;*@blah.com' would be returned as
