@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+import django
 from django.conf import settings
 from django.core.management import call_command
 
@@ -24,9 +25,17 @@ SETTINGS = {
             'NAME': ':memory:',
         }
     },
+    'MIDDLEWARE_CLASSES': (
+        'django.contrib.sessions.middleware.SessionMiddleware',
+        'django.contrib.auth.middleware.AuthenticationMiddleware',
+        'django.contrib.messages.middleware.MessageMiddleware',
+    )
 }
 
 if not settings.configured:
     settings.configure(**SETTINGS)
 
+if django.VERSION >= (1, 7):
+        django.setup()
+
 call_command('test', 'dbsettings')
